'''
Created on 26 avr. 2018

@author: P35746
'''

import types
from datetime import datetime
from pidclient.logging_factory import LoggingFactory
try :
    from airflow import DAG
except ImportError:
    #necessary to allow sphinx generate documentation with airflow is not available
    print("ERROR : Missing required airflow package")
    class DAG():
        def __init__(self):
            raise Exception("airflow import failed")
    
def pre_execute(self, context):
    """method called before the execution of a task

    :param context: the context
    :type context: context
    """
    addendum_information = {}
    if u'task_instance_key_str' in context:
        tmp = str(context[u'task_instance_key_str'])
        tmp = tmp.replace("__", "|")
        values = tmp.split("|")
        addendum_information["dag_id"] = values[0]
        addendum_information["step_id"] = values[1]
        addendum_information["start_date"] = values[2]
    if u'ts' in context:
        addendum_information["start_date"] = str(context[u'ts'])
    self.process_log = LoggingFactory(sysinfo=addendum_information).get_logger("-", "AIRFLOW", datetime.now())
    returned = self.step_started(self.process_log)
    if returned is None:
        pass
    else:
        self.process_log = returned
    self.process_log.proc_started()


def post_execute(self, context, result=None):
    """method called after the execution of a task

    :param context: the context
    :param result: the result
    :type context: context
    :type result: result
    """
    returned = self.step_stopped(self.process_log)
    if returned is None:
        self.process_log.proc_stopped(0)
    else:
        self.process_log = returned


def step_started(self, process_log):
    """method that should be overwritten by the user
    to fix information in the process_log before a step is
    executed

    :param process_log: the process log
    :type process_log: ProcessLog
    """
    return None


def step_stopped(self, process_log):
    """method that should be overwritten by the user
    to fix information in the process_log after a step is
    executed

    :param process_log: the process log
    :type process_log: ProcessLog
    """
    return None


def run(
        self,
        *args, **kwargs):
    """
        method that overiddes the existing method of a dag class
    """
    addendum_information = {}
    addendum_information["dag_id"] = self.dag_id
    addendum_information["start_date"] = str(kwargs['start_date'])
    addendum_information["end_date"] = str(kwargs['end_date'])
    self.process_log = LoggingFactory(sysinfo=addendum_information).get_logger("-", "AIRFLOW", datetime.now())
    returned = self.proc_started(self.process_log)
    if returned is None:
        pass
    else:
        self.process_log = returned
    self.process_log.proc_started()

    self.initrun(*args, **kwargs)

    returned = self.proc_stopped(self.process_log)
    if returned is None:
        self.process_log.proc_stopped(0)
    else:
        self.process_log = returned


def proc_started(self, process_log):
    """method that should be overwritten by the user
    to fix information in the process_log before the driver is
    executed

    :param process_log: the process log
    :type process_log: ProcessLog
    """
    return None


def proc_stopped(self, process_log):
    """method that should be overwritten by the user
    to fix information in the process_log after the driver is
    executed

    :param process_log: the process log
    :type process_log: ProcessLog
    """
    return None


def log_driver(dag, start_step=None, stop_step=None):
    """method that patches the existing dag to log
    driver information

    :param start_step: a function to call before logging the first activity with the process_log as information to let it adapted by the user
    :param stop_step: a function to call at the end of the activity with the process_log as information to let it adapted by the user
    :type start_step: function
    :type stop_step: function
    """
    dag.initrun = dag.run
    dag.run = types.MethodType(run, dag)
    if start_step is None:
        dag.proc_started = types.MethodType(proc_started, dag)
    else:
        dag.proc_started = types.MethodType(start_step, dag)
    if stop_step is None:
        dag.proc_stopped = types.MethodType(proc_stopped, dag)
    else:
        dag.proc_stopped = types.MethodType(stop_step, dag)
    return dag

def log_steps(dag, start_step=None, stop_step=None):
    """method that patches the existing dag to log
    airflow step information

    :param start_step: a function to call before logging the step with the process_log as information to let it adapted by the user
    :param stop_step: a function to call at the end of the step with the process_log as information to let it adapted by the user
    :type start_step: function
    :type stop_step: function
    """
    for task in dag.tasks:
        if start_step is None:
            task.step_started = types.MethodType(step_started, task)
        else:
            task.step_started = types.MethodType(start_step, task)
        if stop_step is None:
            task.step_stopped = types.MethodType(step_stopped, task)
        else:
            task.step_stopped = types.MethodType(stop_step, task)
        task.pre_execute = types.MethodType(pre_execute, task)
        task.post_execute = types.MethodType(post_execute, task)
    return dag

class DAGWithLogging(DAG):
    """Class that extends a DAG to support logging facility"""

    def proc_started(self, process_log):
        """method that should be overwritten by the user
        to fix information in the process_log before the driver is
        executed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        return None

    def proc_stopped(self, process_log):
        """method that should be overwritten by the user
        to fix information in the process_log after the driver is
        executed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        return None

    def log_steps(self, start_step=None, stop_step=None):
        """method that patches the present dag to log
        airflow step information

        :param start_step: a function to call before logging the step with the process_log as information to let it adapted by the user
        :param stop_step: a function to call at the end of the step with the process_log as information to let it adapted by the user
        :type start_step: function
        :type stop_step: function
        """
        for task in self.tasks:
            task.pre_execute = types.MethodType(pre_execute, task)
            task.post_execute = types.MethodType(post_execute, task)
            if start_step is None:
                task.step_started = types.MethodType(step_started, task)
            else:
                task.step_started = types.MethodType(start_step, task)
            if stop_step is None:
                task.step_stopped = types.MethodType(step_stopped, task)
            else:
                task.step_stopped = types.MethodType(stop_step, task)

    def run(
            self,
            *args, **kwargs):
        """
            method that overiddes the existing method of a dag class
        """
        addendum_information = {}
        addendum_information["dag_id"] = self.dag_id
        addendum_information["start_date"] = str(kwargs['start_date'])
        addendum_information["end_date"] = str(kwargs['end_date'])
        self.process_log = LoggingFactory(sysinfo=addendum_information).get_logger("-", "AIRFLOW", datetime.now())
        returned = self.proc_started(self.process_log)
        if returned is None:
            pass
        else:
            self.process_log = returned
        self.process_log.proc_started()

        super(DAGWithLogging, self).run(*args, **kwargs)

        returned = self.proc_stopped(self.process_log)
        if returned is None:
            self.process_log.proc_stopped(0)
        else:
            self.process_log = returned
