"""
Created on 26 april 2018

@author: Tonneau Stephane
"""
import datetime
import random

from pidclient.base_utils import first_not_null
from pidclient.base_utils import Configuration
from pidclient.base_utils import SystemEnvironment
    

DEFAULTS = {"driver_memory": None, "executer_memory": None,
            "min_executors": None, "max_executors": None,
            "num_executers": None, "init_executors": None,
            "queue" : None, "executor_environment" : None}

DRIVER_PTYPE = "airflow_driver"
STEP_PTYPE = "airflow_step"


def as_date(a_date):
    """Function to convert a date provided as a string in a date datatype

    :param a_date : a date provided as a string
    :type a_date : string
    :returns: the provided date converted in a date datatype or None if the date can not be parsed
    :rtype: datetime
    """
    if a_date is None:
        return a_date
    if a_date == "now":
        return datetime.datetime.today()
    format_date = ("%Y-%m-%d %H:%M:%S", "%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%dT%H:%M:%S", "%Y-%m-%dT%H:%M",
                   "%Y-%m-%d", "%d/%m/%Y %H:%M:%S", "%d/%m/%Y %H:%M", "%d/%m/%Y")
    for the_format in format_date:
        try:
            tmp_date = datetime.datetime.strptime(a_date, the_format)
            return tmp_date
        except:
            pass
    return None


class AirflowConfiguration(Configuration):
    """Class that encapsulates the airflow Configuration"""

    def complete(self):
        """
        Complete the configuration information with
        the environments definitions (or default when not available).
        """

        self.driver_memory = first_not_null(self.driver_memory, DEFAULTS["driver_memory"])
        self.executer_memory = first_not_null(self.executer_memory, DEFAULTS["executer_memory"])
        self.min_executors = first_not_null(self.min_executors, DEFAULTS["min_executors"])
        self.max_executors = first_not_null(self.max_executors, DEFAULTS["max_executors"])
        self.num_executers = first_not_null(self.num_executers, DEFAULTS["num_executers"])
        self.init_executors = first_not_null(self.init_executors, DEFAULTS["init_executors"])
        self.queue = first_not_null(self.queue, DEFAULTS["queue"])
        self.executor_environment = first_not_null(
            self.executor_environment, DEFAULTS["executor_environment"])
        return self


class AirflowEnvironment(SystemEnvironment):
    """Class that captures the airflow environment particularities"""

    def __init__(self, **configuration):
        """Initialisator of the class

        :param configuration: specific parameters of the airflow environment
        :type configuration: a dictionary
        """
        super(AirflowEnvironment, self).__init__(**configuration)
        if "step_id" in configuration:
            # This is a slave
            self.job_type = STEP_PTYPE
            ptype = configuration["step_id"]
        else:
            # This is the master
            self.configuration = AirflowConfiguration()
            self.configuration.complete()
            self.job_type = DRIVER_PTYPE
            ptype = "master"
        if ("start_date" in configuration) and ("dag_id" in configuration):
            self.job_id = as_date(configuration['start_date']).strftime("%Y%m%d%H%M%S") + "." + str(configuration['dag_id'])
            self.main_id = as_date(configuration['start_date']).strftime("%Y%m%d%H%M%S") + "." + str(configuration['dag_id']) + "." + ptype
        else:
            self.job_id = datetime.datetime.now().strftime("%Y%m%d%H%M%S") +" ." + str(random.randint(1000000,9999999))
            self.main_id = self.job_id + "." + ptype
        return
