"""
Created on 26 mars 2018

@author: Stephane Tonneau

This module requires the installation of the python kafka library with
    pip install kafka-python

"""
import sys
try:
    import gssapi
except ImportError:
    try:
        sys.path.append('/usr/lib64/python2.7/site-packages/')
        import gssapi
    except ImportError:
        pass
# The next line is required to avoid the import of snappy as VITO
# has also a snappy package from ESA (with different functionalities).
# The side effect of the ESA package is that it induces a 'too many file open'
# issue for kafka producer.
sys.modules['snappy'] = None
try:
    from kafka import KafkaProducer
except ImportError:
    # the unavailibility of the package shall be treated later
    # when trying to use a producer
    try:
        from .kafka import KafkaProducer
    except ImportError:
        pass
from pidclient.base_utils import LoggingMethod


# KAFKABOKERS = ['epod1.vgt.vito.be:6667','epod17.vgt.vito.be:6667']
# KAFKATOPIC = "pid_test"
NODENAME = "kafka"


class KafkaLogging(LoggingMethod):
    """Class that encapsulates the storing of objects in a kafka topic
    for logging purposes.
    """

    def __init__(self, **parameters):
        """Initialisator of the class

        :param configuration: the general logging configuration
        :type configuration: a python configuration object
        """
        super(KafkaLogging, self).__init__(**parameters)
        kafka_brokers = self.get_config(NODENAME, "brokers")
        if kafka_brokers is None:
            self.kafka_brokers = None
            self.logger.error(
                "Missing brokers configuration value for KafkaLogging")
        else:
            self.kafka_brokers = kafka_brokers.replace(" ", "").split(",")
        self.kafka_topic = self.get_config(NODENAME, "topic")
        if self.kafka_topic is None:
            self.logger.error(
                "Missing topic configuration value for KafkaLogging")
        self.secure = self.get_config(NODENAME, "secure")
        try:
            if self.secure is None:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, acks='all')
            else:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, security_protocol='SASL_PLAINTEXT', sasl_mechanism='GSSAPI', acks='all')
            if self.producer is None:
                self.logger.error(
                    "Kafka logging : kafka producer NOT CREATED !!!")
        except:
            self.logger.error(
                "LOGGING ISSUE : unable to create a kafka producer")
            self.producer = None

    def close(self):
        """Close the logging instance"""
        if self.producer is not None:
            try:
                self.producer.flush()
                self.producer.close()
            except:
                self.logger.error(
                    "LOGGING ISSUE : unable to close kafka system")
            self.producer = None

    def process_log(self, entry, meta_information):
        """Method to insert a json representation of both an object
        and its metadata in a kafka topic

        :param entry: the standard information to log
        :param meta_information: meta information about the information to log
        :type entry: PidEntry
        :type meta_information: Metainformation
        """
        res = None
        if self.producer is not None:
            try:
                entry.id = meta_information.uid
                future = self.producer.send(
                    self.kafka_topic, str(entry.to_json()))
                res = future.get(timeout=60)
            except:
                self.logger.error(
                    "LOGGING ISSUE : unable to process the logging entry with kafka")
        return res
